// FILE:   Lab4_cpu01.c

#include "F28x_Project.h"

// Function Prototypes
void ConfigureADC(void);
void ConfigureEPWM(void);
void ConfigureDAC(void);
void SetupADCEpwm(void);
interrupt void adca1_isr(void);
interrupt void ipc1_isr(void);

// Variables
Uint16 dacbOutput;
Uint16 AdcaResult;


void main(void)
{
// Initialize System Control
	InitSysCtrl();
    EALLOW;
    ClkCfgRegs.PERCLKDIVSEL.bit.EPWMCLKDIV = 1;
    EDIS;

// Initialize GPIO
	InitGpio();  

	EALLOW;
	GpioCtrlRegs.GPADIR.bit.GPIO31 = 1;		// Drives LED LD2 on controlCARD
	EDIS;
	GpioDataRegs.GPADAT.bit.GPIO31 = 1;		// Turn off LED
    
// Clear all interrupts and initialize PIE control registers
	DINT;
	InitPieCtrl();

// Disable CPU interrupts and clear all CPU interrupt flags
	IER = 0x0000;
	IFR = 0x0000;

// Initialize the PIE vector table
	InitPieVectTable();

// Map ISR functions
	EALLOW;
	PieVectTable.ADCA1_INT = &adca1_isr; 	// ADCA1 interrupt
	PieVectTable.IPC0_INT = &ipc1_isr;		// IPC1 interrupt
	EDIS;

// Configure the ADC and power it up
	ConfigureADC();

// Configure the ePWM
	ConfigureEPWM();

// Configure DAC-B
    ConfigureDAC();

// Setup the ADC for ePWM triggered conversions on channel 0
	SetupADCEpwm();

	IpcRegs.IPCCLR.all = 0xFFFFFFFF;

// Enable PIE interrupts
    PieCtrlRegs.PIEIER1.bit.INTx1 = 1;	    // ADC ISR
    PieCtrlRegs.PIEIER1.bit.INTx13 = 1;	    // IPC1 ISR

// Enable global interrupts and higher priority real-time debug events:
	IER |= M_INT1; 						// Enable group 1 interrupts
	EINT;  								// Enable Global interrupt INTM
	ERTM;  								// Enable Global realtime interrupt DBGM

// Wait here until CPU02 is ready
	while (IpcRegs.IPCSTS.bit.IPC17 == 0) ;	// Wait for CPU02 to set IPC17
	IpcRegs.IPCACK.bit.IPC17 = 1;			// Acknowledge and clear IPC17


// Sync and start ePWM
    EALLOW;
    CpuSysRegs.PCLKCR0.bit.TBCLKSYNC = 1;
    EPwm2Regs.ETSEL.bit.SOCAEN = 1; 	// Enable SOCA
    EPwm2Regs.TBCTL.bit.CTRMODE = 0; 	// Un-freeze, and enter up count mode
    EDIS;

// main loop
	while(1)
	{
      	GpioDataRegs.GPADAT.bit.GPIO31 = 0;		// Turn on LED
      	DELAY_US(1000 * 500);					// ON delay
      	GpioDataRegs.GPADAT.bit.GPIO31 = 1;   	// Turn off LED
      	DELAY_US(1000 * 500);					// OFF delay
	}
}


// Write ADC configurations and power up the ADC for both ADC A and ADC B
void ConfigureADC(void)
{
	EALLOW;
	AdcaRegs.ADCCTL2.bit.PRESCALE = 6; 			// Set ADCCLK divider to /4
	AdcaRegs.ADCCTL2.bit.RESOLUTION = 0;	 	// 12-bit resolution
	AdcaRegs.ADCCTL2.bit.SIGNALMODE = 0; 		// Single-ended channel conversions (12-bit mode only)
	AdcaRegs.ADCCTL1.bit.INTPULSEPOS = 1;		// Set pulse positions to late
	AdcaRegs.ADCCTL1.bit.ADCPWDNZ = 1;			// Power up the ADC
	DELAY_US(1000);								// Delay for 1ms to allow ADC time to power up
	EDIS;
}


void ConfigureEPWM(void)
{
	EALLOW;
	// Assumes ePWM clock is already enabled
	EPwm2Regs.TBCTL.bit.CTRMODE = 3;            // Freeze counter
	EPwm2Regs.TBCTL.bit.HSPCLKDIV = 0;			// TBCLK pre-scaler = /1
	EPwm2Regs.TBPRD = 0x07D0;			        // Set period to 2000 counts (50kHz)
	EPwm2Regs.ETSEL.bit.SOCAEN	= 0;	        // Disable SOC on A group
	EPwm2Regs.ETSEL.bit.SOCASEL	= 2;	        // Select SOCA on period match
    EPwm2Regs.ETSEL.bit.SOCAEN = 1; 			// Enable SOCA
	EPwm2Regs.ETPS.bit.SOCAPRD = 1;		        // Generate pulse on 1st event
	EDIS;
}


void ConfigureDAC(void)
{
	// Configure DACB
	EALLOW;
	DacbRegs.DACCTL.bit.DACREFSEL = 1;			// Use ADC references
	DacbRegs.DACCTL.bit.LOADMODE = 0;			// Load on next SYSCLK
	DacbRegs.DACVALS.all = 0x0800;				// Set mid-range
	DacbRegs.DACOUTEN.bit.DACOUTEN = 1;			// Enable DAC
	EDIS;
}


void SetupADCEpwm(void)
{
	EALLOW;
	AdcaRegs.ADCSOC0CTL.bit.CHSEL = 0;  		// SOC0 will convert pin A0
	AdcaRegs.ADCSOC0CTL.bit.ACQPS = 14; 		// Sample window is 100 SYSCLK cycles
	AdcaRegs.ADCSOC0CTL.bit.TRIGSEL = 7; 		// Trigger on ePWM2 SOCA/C
	AdcaRegs.ADCINTSEL1N2.bit.INT1SEL = 0; 		// End of SOC0 will set INT1 flag
	AdcaRegs.ADCINTSEL1N2.bit.INT1E = 1;   		// Enable INT1 flag
	AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1; 		// Make sure INT1 flag is cleared
	EDIS;
}


interrupt void ipc1_isr(void)
{
	// Read IPC channel 0 and write to DACB
	dacbOutput = (Uint16) IpcRegs.IPCRECVADDR;	// Read data on IPC ADDR register
	DacbRegs.DACVALS.all = dacbOutput;			// Write to DACB

	// Return from interrupt
	IpcRegs.IPCACK.bit.IPC0 = 1;				// Clear IPC1 bit
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;		// Acknowledge PIE group 1 to enable further interrupts
}


interrupt void adca1_isr(void)
{
	// Read the ADC result and write to IPC channel 2
	AdcaResult = AdcaResultRegs.ADCRESULT0;
	IpcRegs.IPCSENDDATA = (Uint32) AdcaResult;	// Write the result to the IPC data register
	IpcRegs.IPCSET.bit.IPC1 = 1;				// Set the IPC2 flag for CPU2

	// Return from interrupt
	AdcaRegs.ADCINTFLGCLR.bit.ADCINT1 = 1; 		// Clear ADC INT1 flag
	PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;		// Acknowledge PIE group 1 to enable further interrupts
}

 // end of file
